/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.AndFileFilter;
import org.apache.commons.io.filefilter.PrefixFileFilter;
import org.apache.commons.io.filefilter.SuffixFileFilter;
import org.apache.commons.io.filefilter.TrueFileFilter;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Unit test about the legal information that the archive must contain
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class LegalIT {

    private final static String PROPERTY_NAME_ARCHIVE_BASE_DIRECTORY = "archive-base-directory";

    private static File ARCHIVE_BASE_DIRECTORY = null;

    @BeforeClass
    public static void setup() {
        final String archiveBaseDirectoryStr = System.getProperty(PROPERTY_NAME_ARCHIVE_BASE_DIRECTORY);
        assertNotNull("System property '" + PROPERTY_NAME_ARCHIVE_BASE_DIRECTORY + "' not set.",
                archiveBaseDirectoryStr);
        ARCHIVE_BASE_DIRECTORY = new File(archiveBaseDirectoryStr);
        assertTrue("The archive base directory '" + ARCHIVE_BASE_DIRECTORY.getAbsolutePath() + "' does not exist",
                ARCHIVE_BASE_DIRECTORY.exists());
        assertTrue("The archive base directory '" + ARCHIVE_BASE_DIRECTORY.getAbsolutePath() + "' is not a directory",
                ARCHIVE_BASE_DIRECTORY.isDirectory());
    }

    /**
     * <p>
     * Verify that a file 'LICENSE' exists.
     * </p>
     */
    @Test
    public void licenseFile() throws IOException {
        
        final File licenseFile = new File(ARCHIVE_BASE_DIRECTORY, "LICENSE");
        assertTrue("'LICENSE' file missing", FileUtils.directoryContains(ARCHIVE_BASE_DIRECTORY, licenseFile));
        assertTrue("Too short file 'LICENSE'", FileUtils.sizeOf(licenseFile) > 20000);
    }

    /**
     * <p>
     * Verify that a file 'THIRD-PARTY' exists.
     * </p>
     */
    @Test
    public void thirdPartyFile() throws IOException {

        final File thirdPartyFile = new File(ARCHIVE_BASE_DIRECTORY, "THIRD-PARTY");
        assertTrue("'THIRD-PARTY' file missing", FileUtils.directoryContains(ARCHIVE_BASE_DIRECTORY, thirdPartyFile));
        assertTrue("Too short file 'THIRD-PARTY'", FileUtils.sizeOf(thirdPartyFile) > 2000);
    }

    /**
     * <p>
     * Verify that license files of third party are packaged.
     * </p>
     */
    @Test
    public void thirdPartyLicenseFiles() throws IOException {

        final File thirdPartyLIcenseFiles = new File(ARCHIVE_BASE_DIRECTORY, "licenses");
        assertTrue("3PP licenses missing", FileUtils.directoryContains(ARCHIVE_BASE_DIRECTORY, thirdPartyLIcenseFiles));
        assertTrue("Not enough different 3PP licenses", thirdPartyLIcenseFiles.listFiles().length > 3);
        assertTrue("Too many different 3PP licenses", thirdPartyLIcenseFiles.listFiles().length < 5);
    }

    /**
     * <p>
     * Verify that a file 'COPYRIGHT' exists.
     * </p>
     */
    @Test
    public void copyrightFile() throws IOException {

        final File copyrightFile = new File(ARCHIVE_BASE_DIRECTORY, "COPYRIGHT");
        assertTrue("'COPYRIGHT' file missing", FileUtils.directoryContains(ARCHIVE_BASE_DIRECTORY, copyrightFile));
        assertTrue("Too short file 'COPYRIGHT'", FileUtils.sizeOf(copyrightFile) > 500);
        assertTrue("Too long file 'COPYRIGHT'", FileUtils.sizeOf(copyrightFile) < 1000);
    }

    /**
     * <p>
     * Verify that all our Jar files contains the files 'COPYRIGHT' and 'LICENSE'.
     * </p>
     */
    @Test
    public void copyrightAndLicenseFilesInJars() throws IOException {

        final Collection<File> jarFiles = FileUtils.listFiles(ARCHIVE_BASE_DIRECTORY,
                new AndFileFilter(new PrefixFileFilter(new String[] { "easycommons-", "easywsdl-", "petals-" }),
                        new SuffixFileFilter(".jar")),
                TrueFileFilter.INSTANCE);

        for (final File jarFile : jarFiles) {
            final JarInputStream jarIO = new JarInputStream(new FileInputStream(jarFile));
            try {
                boolean copyrightFound = false;
                boolean licenseFound = false;
                JarEntry jarEntry;
                while ((jarEntry = jarIO.getNextJarEntry()) != null) {
                    if (!jarEntry.isDirectory()) {
                        if ("META-INF/COPYRIGHT".equals(jarEntry.getName())) {
                            copyrightFound = true;
                        } else if ("META-INF/LICENSE".equals(jarEntry.getName())) {
                            licenseFound = true;
                        }
                    }
                }
                assertTrue("File 'META-INF/COPYRIGHT' not found", copyrightFound);
                assertTrue("File 'META-INF/LICENSE' not found", licenseFound);
            } finally {
                jarIO.close();
            }
        }
    }

}
